# pylint: disable=too-many-lines
"""The wfmoutpre commands module.

These commands are used in the following models:
DPO4K, DPO4KB, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - WFMOutpre:BIT_Nr <NR1>
    - WFMOutpre:BIT_Nr?
    - WFMOutpre:BN_Fmt {RI|RP|FP}
    - WFMOutpre:BN_Fmt?
    - WFMOutpre:BYT_Nr <NR1>
    - WFMOutpre:BYT_Nr?
    - WFMOutpre:BYT_Or {LSB|MSB}
    - WFMOutpre:BYT_Or?
    - WFMOutpre:CENTERFREQuency?
    - WFMOutpre:DOMain?
    - WFMOutpre:ENCdg {ASCii|BINary}
    - WFMOutpre:ENCdg?
    - WFMOutpre:NR_Pt?
    - WFMOutpre:PT_Fmt?
    - WFMOutpre:PT_ORder?
    - WFMOutpre:PT_Off?
    - WFMOutpre:REFLEvel?
    - WFMOutpre:SPAN?
    - WFMOutpre:WFId?
    - WFMOutpre:WFMTYPe?
    - WFMOutpre:XINcr?
    - WFMOutpre:XUNit?
    - WFMOutpre:XZEro?
    - WFMOutpre:YMUlt?
    - WFMOutpre:YOFf?
    - WFMOutpre:YUNit?
    - WFMOutpre:YZEro?
    - WFMOutpre?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class WfmoutpreYzero(SCPICmdRead):
    """The ``WFMOutpre:YZEro`` command.

    **Description:**
        - This query-only command returns the combined vertical position and offset for the source
          waveform specified by ``DATA:SOURCE``. This represents a departure from previous
          instruments where the ``:YZEro`` value represented the vertical position in vertical units
          and the ``:YOFf`` value represented the vertical offset in digitizing levels. For this
          instrument family, the value of ``:YOFf`` is always 0.0. An error is reported if the
          ``DATA:SOURCE`` waveform does not exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:YZEro?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:YZEro?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:YZEro?
    """


class WfmoutpreYunit(SCPICmdRead):
    """The ``WFMOutpre:YUNit`` command.

    **Description:**
        - This query-only command returns the vertical units for the waveform specified by the
          ``DATA:SOURCE`` command. An error is reported if the ``DATa:SOUrce`` waveform does not
          exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:YUNit?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:YUNit?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:YUNit?
    """


class WfmoutpreYoff(SCPICmdRead):
    """The ``WFMOutpre:YOFf`` command.

    **Description:**
        - This query-only command returns the vertical offset of the source specified by
          ``DATA:SOURCE``. For this instrument family, the value returned is always 0.0 as the
          offset is combined with the ``:YZEro`` value. An error is reported if the ``DATA:SOURCE``
          waveform does not exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:YOFf?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:YOFf?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:YOFf?
    """


class WfmoutpreYmult(SCPICmdRead):
    """The ``WFMOutpre:YMUlt`` command.

    **Description:**
        - This query-only command returns the vertical scale factor per digitizing level in units
          specified by ``WFMOutpre:YUNit`` for the waveform specified by the ``DATA:SOURCE``
          command. For those formats in which ``WFMOUTPRE:BYT_NR`` is important (all non-floating
          point formats), ``WFMOutpre:YMUlt?`` must take the location of the binary point implied by
          ``BYT_NR`` into consideration. An error is reported if the ``DATa:SOUrce`` waveform does
          not exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:YMUlt?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:YMUlt?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:YMUlt?
    """


class WfmoutpreXzero(SCPICmdRead):
    """The ``WFMOutpre:XZEro`` command.

    **Description:**
        - This query-only command returns the sub-sample time between the trigger sample (designated
          by ``PT_OFF``) and the occurrence of the actual trigger for the waveform specified by the
          ``DATA:SOURCE`` command. This value is in units of ``WFMOutpre:XUNit``. An error is
          reported if the ``DATa:SOUrce`` waveform does not exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:XZEro?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:XZEro?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:XZEro?
    """


class WfmoutpreXunit(SCPICmdRead):
    """The ``WFMOutpre:XUNit`` command.

    **Description:**
        - This query-only command returns the horizontal units for the waveform specified by the
          ``DATA:SOURCE`` command. An error is reported if the ``DATa:SOUrce`` waveform does not
          exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:XUNit?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:XUNit?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:XUNit?
    """


class WfmoutpreXincr(SCPICmdRead):
    """The ``WFMOutpre:XINcr`` command.

    **Description:**
        - This query-only command returns the horizontal point spacing in units of
          ``WFMOutpre:XUNit`` for the waveform specified by the ``DATA:SOURCE`` command. This value
          corresponds to the sampling interval. An error is reported if the ``DATa:SOUrce`` waveform
          does not exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:XINcr?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:XINcr?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:XINcr?
    """


class WfmoutpreWfmtype(SCPICmdRead):
    """The ``WFMOutpre:WFMTYPe`` command.

    **Description:**
        - This query returns the type of the outgoing waveform. The type of waveform that is being
          transferred in turn determines in which window it will display on the instrument screen:
          (the Waveform View window or Spectrum View window). Both the Analog and ``RF_TD``
          arguments specify the time domain window; the ``RF_RD`` argument specifies the frequency
          domain window.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:WFMTYPe?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:WFMTYPe?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:WFMTYPe?
    """


class WfmoutpreWfid(SCPICmdRead):
    """The ``WFMOutpre:WFId`` command.

    **Description:**
        - This query-only command returns a string describing several aspects of the acquisition
          parameters for the waveform specified by the ``DATA:SOURCE`` command. An error is reported
          if the ``DATa:SOUrce`` waveform does not exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:WFId?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:WFId?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:WFId?
    """


class WfmoutpreSpan(SCPICmdRead):
    """The ``WFMOutpre:SPAN`` command.

    **Description:**
        - This query returns the frequency span of the outgoing waveform. The span is the range of
          frequencies you can observe around the center frequency.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:SPAN?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:SPAN?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:SPAN?
    """


class WfmoutpreReflevel(SCPICmdRead):
    """The ``WFMOutpre:REFLEvel`` command.

    **Description:**
        - This query returns the Reference Level of the outgoing waveform. It applies only to the
          four frequency domain waveforms (RF Normal, RF Average, RF Max Hold, and RF Min Hold). The
          units are in watts.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:REFLEvel?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:REFLEvel?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:REFLEvel?
    """


class WfmoutprePtOff(SCPICmdRead):
    """The ``WFMOutpre:PT_Off`` command.

    **Description:**
        - This query-only command returns the trigger point relative to ``DATA:START`` for the
          waveform specified by the ``DATA:SOURCE`` command.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:PT_Off?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_Off?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:PT_Off?
    """


class WfmoutprePtOrder(SCPICmdRead):
    """The ``WFMOutpre:PT_ORder`` command.

    **Description:**
        - This query-only command specifies whether the source waveform is Fast Acquisition. A Fast
          Acquisition waveform is stored as a 500 (vertical) by 1000 (horizontal) point bitmap. Each
          point represents display intensity for that screen location. Only CURVe? query functions
          are allowed on Fast Acquisition waveforms. When the ``WFMOutpre:PT_ORder?`` query returns
          Row, this indicates that the source is a Fast Acquisition waveform (and that each of 1000
          possible horizontal columns being transmitted contains 500 vertical points. Note: 500
          might vary based on how many channels enabled from same FastAcq group.) and waveform
          points are transmitted in the following order: left to right; then top to bottom. When the
          ``WFMOutpre:PT_ORder?`` query returns Linear, this indicates that the source is not a Fast
          Acquisition waveform (and that each horizontal column being sent contains only one
          vertical point). Note that waveform points are transmitted in the following order: top to
          bottom, then left to right.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:PT_ORder?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_ORder?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:PT_ORder?
    """


class WfmoutprePtFmt(SCPICmdRead):
    """The ``WFMOutpre:PT_Fmt`` command.

    **Description:**
        - This query-only command returns the point format for the waveform specified by the
          ``DATA:SOURCE`` command. The format specifies a set of equations describing how the scale
          factors in the preamble are used to give meaning to the CURVE data points. An error is
          reported if the ``DATa:SOUrce`` waveform does not exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:PT_Fmt?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_Fmt?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:PT_Fmt?
    """


class WfmoutpreNrPt(SCPICmdRead):
    """The ``WFMOutpre:NR_Pt`` command.

    **Description:**
        - This query-only command returns the number of points for the ``DATA:SOURCE`` waveform that
          will be transmitted in response to a CURVE? query.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:NR_Pt?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:NR_Pt?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:NR_Pt?
    """


class WfmoutpreEncdg(SCPICmdWrite, SCPICmdRead):
    """The ``WFMOutpre:ENCdg`` command.

    **Description:**
        - This command sets or queries the type of encoding for outgoing waveforms.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:ENCdg?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:ENCdg?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMOutpre:ENCdg value`` command.

    **SCPI Syntax:**

    ::

        - WFMOutpre:ENCdg {ASCii|BINary}
        - WFMOutpre:ENCdg?

    **Info:**
        - ``ASCii`` specifies that the outgoing data is to be in ASCII format. Waveforms internally
          stored as integers will be sent as <NR1> numbers, while those stored as floating point
          will be sent as <NR3> numbers.
        - ``BINary`` specifies that outgoing data is to be in a binary format whose further
          specification is determined by ``WFMOUTPRE:BYT_NR``, ``WFMOUTPRE:BIT_NR``,
          ``WFMOUTPRE:BN_FMT`` and ``WFMOUTPRE:BYT_OR``.
    """


class WfmoutpreDomain(SCPICmdRead):
    """The ``WFMOutpre:DOMain`` command.

    **Description:**
        - This query returns the domain of the outgoing waveform, either TIMe or FREQuency. If the
          domain is TIMe, it indicates that the data is to be treated as integer information. If the
          domain is FREQuency, it indicates that the data is to be treated as floating point
          information.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:DOMain?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:DOMain?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:DOMain?
    """


class WfmoutpreCenterfrequency(SCPICmdRead):
    """The ``WFMOutpre:CENTERFREQuency`` command.

    **Description:**
        - This query returns the center frequency of the outgoing waveform. The center frequency is
          a precise frequency at the center of the display of the frequency domain graticule. In
          many applications, it is a carrier frequency.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:CENTERFREQuency?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:CENTERFREQuency?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:CENTERFREQuency?
    """


class WfmoutpreBytOr(SCPICmdWrite, SCPICmdRead):
    """The ``WFMOutpre:BYT_Or`` command.

    **Description:**
        - This command sets or queries which byte of binary waveform data is transmitted first,
          during a waveform data transfer, when data points require more than one byte. This
          specification only has meaning when ``WFMOUTPRE:ENCDG`` is set to BIN.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:BYT_Or?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:BYT_Or?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMOutpre:BYT_Or value`` command.

    **SCPI Syntax:**

    ::

        - WFMOutpre:BYT_Or {LSB|MSB}
        - WFMOutpre:BYT_Or?

    **Info:**
        - ``LSB`` specifies that the least significant byte will be transmitted first.
        - ``MSB`` specifies that the most significant byte will be transmitted first.
    """


class WfmoutpreBytNr(SCPICmdWrite, SCPICmdRead):
    """The ``WFMOutpre:BYT_Nr`` command.

    **Description:**
        - This command sets or queries the binary field data width (bytes per point) for the
          waveform specified by the ``DATA:SOURCE`` command. Note that values will be constrained
          according to the underlying waveform data. This specification is only meaningful when
          ``WFMOUTPRE:ENCDG`` is set to BIN, and ``WFMOUTPRE:BN_FMT`` is set to either RI or RP.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:BYT_Nr?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:BYT_Nr?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMOutpre:BYT_Nr value`` command.

    **SCPI Syntax:**

    ::

        - WFMOutpre:BYT_Nr <NR1>
        - WFMOutpre:BYT_Nr?

    **Info:**
        - ``<NR1>`` is the number of bytes per data point and can be 1, 2 or 8. A value of 1 or 2
          bytes per waveform point indicates channel data; 8 bytes per waveform point indicate pixel
          map (fast acquisition) data.
    """


class WfmoutpreBnFmt(SCPICmdWrite, SCPICmdRead):
    """The ``WFMOutpre:BN_Fmt`` command.

    **Description:**
        - This command sets or queries the format of binary data for outgoing waveforms specified by
          the ``DATA:SOURCE`` command.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:BN_Fmt?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:BN_Fmt?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMOutpre:BN_Fmt value`` command.

    **SCPI Syntax:**

    ::

        - WFMOutpre:BN_Fmt {RI|RP|FP}
        - WFMOutpre:BN_Fmt?

    **Info:**
        - ``RI`` specifies signed integer data point representation.
        - ``RP`` specifies positive integer data point representation.
        - ``FP`` specifies floating point representation.
    """


class WfmoutpreBitNr(SCPICmdWrite, SCPICmdRead):
    """The ``WFMOutpre:BIT_Nr`` command.

    **Description:**
        - This command sets and queries the number of bits per waveform point that outgoing
          waveforms contain, as specified by the ``DATA:SOURCE`` command. Note that values will be
          constrained according to the underlying waveform data. This specification is only
          meaningful when ``WFMOUTPRE:ENCDG`` is set to BIN and ``WFMOUTPRE:BN_FMT`` is set to
          either RI or RP.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:BIT_Nr?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:BIT_Nr?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMOutpre:BIT_Nr value`` command.

    **SCPI Syntax:**

    ::

        - WFMOutpre:BIT_Nr <NR1>
        - WFMOutpre:BIT_Nr?

    **Info:**
        - ``<NR1>`` number of bits per data point can be 8 or 16.
    """


#  pylint: disable=too-many-instance-attributes,too-many-public-methods
class Wfmoutpre(SCPICmdRead):
    """The ``WFMOutpre`` command.

    **Description:**
        - This query-only command queries the waveform formatting data for the waveform specified by
          the ``DATA:SOURCE`` command. The preamble components are considered to be of two types;
          formatting and interpretation. The formatting components are: ENCdg, ``BN_Fmt``,
          ``BYT_Or``, ``BYT_Nr``, ``BIT_Nr``. The interpretation components are derived from the
          ``DATa:SOUrce`` specified waveform.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre?

    Properties:
        - ``.bit_nr``: The ``WFMOutpre:BIT_Nr`` command.
        - ``.bn_fmt``: The ``WFMOutpre:BN_Fmt`` command.
        - ``.byt_nr``: The ``WFMOutpre:BYT_Nr`` command.
        - ``.byt_or``: The ``WFMOutpre:BYT_Or`` command.
        - ``.centerfrequency``: The ``WFMOutpre:CENTERFREQuency`` command.
        - ``.domain``: The ``WFMOutpre:DOMain`` command.
        - ``.encdg``: The ``WFMOutpre:ENCdg`` command.
        - ``.nr_pt``: The ``WFMOutpre:NR_Pt`` command.
        - ``.pt_fmt``: The ``WFMOutpre:PT_Fmt`` command.
        - ``.pt_order``: The ``WFMOutpre:PT_ORder`` command.
        - ``.pt_off``: The ``WFMOutpre:PT_Off`` command.
        - ``.reflevel``: The ``WFMOutpre:REFLEvel`` command.
        - ``.span``: The ``WFMOutpre:SPAN`` command.
        - ``.wfid``: The ``WFMOutpre:WFId`` command.
        - ``.wfmtype``: The ``WFMOutpre:WFMTYPe`` command.
        - ``.xincr``: The ``WFMOutpre:XINcr`` command.
        - ``.xunit``: The ``WFMOutpre:XUNit`` command.
        - ``.xzero``: The ``WFMOutpre:XZEro`` command.
        - ``.ymult``: The ``WFMOutpre:YMUlt`` command.
        - ``.yoff``: The ``WFMOutpre:YOFf`` command.
        - ``.yunit``: The ``WFMOutpre:YUNit`` command.
        - ``.yzero``: The ``WFMOutpre:YZEro`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "WFMOutpre") -> None:
        super().__init__(device, cmd_syntax)
        self._bit_nr = WfmoutpreBitNr(device, f"{self._cmd_syntax}:BIT_Nr")
        self._bn_fmt = WfmoutpreBnFmt(device, f"{self._cmd_syntax}:BN_Fmt")
        self._byt_nr = WfmoutpreBytNr(device, f"{self._cmd_syntax}:BYT_Nr")
        self._byt_or = WfmoutpreBytOr(device, f"{self._cmd_syntax}:BYT_Or")
        self._centerfrequency = WfmoutpreCenterfrequency(
            device, f"{self._cmd_syntax}:CENTERFREQuency"
        )
        self._domain = WfmoutpreDomain(device, f"{self._cmd_syntax}:DOMain")
        self._encdg = WfmoutpreEncdg(device, f"{self._cmd_syntax}:ENCdg")
        self._nr_pt = WfmoutpreNrPt(device, f"{self._cmd_syntax}:NR_Pt")
        self._pt_fmt = WfmoutprePtFmt(device, f"{self._cmd_syntax}:PT_Fmt")
        self._pt_order = WfmoutprePtOrder(device, f"{self._cmd_syntax}:PT_ORder")
        self._pt_off = WfmoutprePtOff(device, f"{self._cmd_syntax}:PT_Off")
        self._reflevel = WfmoutpreReflevel(device, f"{self._cmd_syntax}:REFLEvel")
        self._span = WfmoutpreSpan(device, f"{self._cmd_syntax}:SPAN")
        self._wfid = WfmoutpreWfid(device, f"{self._cmd_syntax}:WFId")
        self._wfmtype = WfmoutpreWfmtype(device, f"{self._cmd_syntax}:WFMTYPe")
        self._xincr = WfmoutpreXincr(device, f"{self._cmd_syntax}:XINcr")
        self._xunit = WfmoutpreXunit(device, f"{self._cmd_syntax}:XUNit")
        self._xzero = WfmoutpreXzero(device, f"{self._cmd_syntax}:XZEro")
        self._ymult = WfmoutpreYmult(device, f"{self._cmd_syntax}:YMUlt")
        self._yoff = WfmoutpreYoff(device, f"{self._cmd_syntax}:YOFf")
        self._yunit = WfmoutpreYunit(device, f"{self._cmd_syntax}:YUNit")
        self._yzero = WfmoutpreYzero(device, f"{self._cmd_syntax}:YZEro")

    @property
    def bit_nr(self) -> WfmoutpreBitNr:
        """Return the ``WFMOutpre:BIT_Nr`` command.

        **Description:**
            - This command sets and queries the number of bits per waveform point that outgoing
              waveforms contain, as specified by the ``DATA:SOURCE`` command. Note that values will
              be constrained according to the underlying waveform data. This specification is only
              meaningful when ``WFMOUTPRE:ENCDG`` is set to BIN and ``WFMOUTPRE:BN_FMT`` is set to
              either RI or RP.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:BIT_Nr?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:BIT_Nr?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMOutpre:BIT_Nr value`` command.

        **SCPI Syntax:**

        ::

            - WFMOutpre:BIT_Nr <NR1>
            - WFMOutpre:BIT_Nr?

        **Info:**
            - ``<NR1>`` number of bits per data point can be 8 or 16.
        """
        return self._bit_nr

    @property
    def bn_fmt(self) -> WfmoutpreBnFmt:
        """Return the ``WFMOutpre:BN_Fmt`` command.

        **Description:**
            - This command sets or queries the format of binary data for outgoing waveforms
              specified by the ``DATA:SOURCE`` command.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:BN_Fmt?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:BN_Fmt?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMOutpre:BN_Fmt value`` command.

        **SCPI Syntax:**

        ::

            - WFMOutpre:BN_Fmt {RI|RP|FP}
            - WFMOutpre:BN_Fmt?

        **Info:**
            - ``RI`` specifies signed integer data point representation.
            - ``RP`` specifies positive integer data point representation.
            - ``FP`` specifies floating point representation.
        """
        return self._bn_fmt

    @property
    def byt_nr(self) -> WfmoutpreBytNr:
        """Return the ``WFMOutpre:BYT_Nr`` command.

        **Description:**
            - This command sets or queries the binary field data width (bytes per point) for the
              waveform specified by the ``DATA:SOURCE`` command. Note that values will be
              constrained according to the underlying waveform data. This specification is only
              meaningful when ``WFMOUTPRE:ENCDG`` is set to BIN, and ``WFMOUTPRE:BN_FMT`` is set to
              either RI or RP.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:BYT_Nr?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:BYT_Nr?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMOutpre:BYT_Nr value`` command.

        **SCPI Syntax:**

        ::

            - WFMOutpre:BYT_Nr <NR1>
            - WFMOutpre:BYT_Nr?

        **Info:**
            - ``<NR1>`` is the number of bytes per data point and can be 1, 2 or 8. A value of 1 or
              2 bytes per waveform point indicates channel data; 8 bytes per waveform point indicate
              pixel map (fast acquisition) data.
        """
        return self._byt_nr

    @property
    def byt_or(self) -> WfmoutpreBytOr:
        """Return the ``WFMOutpre:BYT_Or`` command.

        **Description:**
            - This command sets or queries which byte of binary waveform data is transmitted first,
              during a waveform data transfer, when data points require more than one byte. This
              specification only has meaning when ``WFMOUTPRE:ENCDG`` is set to BIN.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:BYT_Or?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:BYT_Or?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMOutpre:BYT_Or value`` command.

        **SCPI Syntax:**

        ::

            - WFMOutpre:BYT_Or {LSB|MSB}
            - WFMOutpre:BYT_Or?

        **Info:**
            - ``LSB`` specifies that the least significant byte will be transmitted first.
            - ``MSB`` specifies that the most significant byte will be transmitted first.
        """
        return self._byt_or

    @property
    def centerfrequency(self) -> WfmoutpreCenterfrequency:
        """Return the ``WFMOutpre:CENTERFREQuency`` command.

        **Description:**
            - This query returns the center frequency of the outgoing waveform. The center frequency
              is a precise frequency at the center of the display of the frequency domain graticule.
              In many applications, it is a carrier frequency.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:CENTERFREQuency?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:CENTERFREQuency?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:CENTERFREQuency?
        """
        return self._centerfrequency

    @property
    def domain(self) -> WfmoutpreDomain:
        """Return the ``WFMOutpre:DOMain`` command.

        **Description:**
            - This query returns the domain of the outgoing waveform, either TIMe or FREQuency. If
              the domain is TIMe, it indicates that the data is to be treated as integer
              information. If the domain is FREQuency, it indicates that the data is to be treated
              as floating point information.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:DOMain?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:DOMain?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:DOMain?
        """
        return self._domain

    @property
    def encdg(self) -> WfmoutpreEncdg:
        """Return the ``WFMOutpre:ENCdg`` command.

        **Description:**
            - This command sets or queries the type of encoding for outgoing waveforms.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:ENCdg?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:ENCdg?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMOutpre:ENCdg value`` command.

        **SCPI Syntax:**

        ::

            - WFMOutpre:ENCdg {ASCii|BINary}
            - WFMOutpre:ENCdg?

        **Info:**
            - ``ASCii`` specifies that the outgoing data is to be in ASCII format. Waveforms
              internally stored as integers will be sent as <NR1> numbers, while those stored as
              floating point will be sent as <NR3> numbers.
            - ``BINary`` specifies that outgoing data is to be in a binary format whose further
              specification is determined by ``WFMOUTPRE:BYT_NR``, ``WFMOUTPRE:BIT_NR``,
              ``WFMOUTPRE:BN_FMT`` and ``WFMOUTPRE:BYT_OR``.
        """
        return self._encdg

    @property
    def nr_pt(self) -> WfmoutpreNrPt:
        """Return the ``WFMOutpre:NR_Pt`` command.

        **Description:**
            - This query-only command returns the number of points for the ``DATA:SOURCE`` waveform
              that will be transmitted in response to a CURVE? query.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:NR_Pt?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:NR_Pt?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:NR_Pt?
        """
        return self._nr_pt

    @property
    def pt_fmt(self) -> WfmoutprePtFmt:
        """Return the ``WFMOutpre:PT_Fmt`` command.

        **Description:**
            - This query-only command returns the point format for the waveform specified by the
              ``DATA:SOURCE`` command. The format specifies a set of equations describing how the
              scale factors in the preamble are used to give meaning to the CURVE data points. An
              error is reported if the ``DATa:SOUrce`` waveform does not exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:PT_Fmt?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_Fmt?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:PT_Fmt?
        """
        return self._pt_fmt

    @property
    def pt_order(self) -> WfmoutprePtOrder:
        """Return the ``WFMOutpre:PT_ORder`` command.

        **Description:**
            - This query-only command specifies whether the source waveform is Fast Acquisition. A
              Fast Acquisition waveform is stored as a 500 (vertical) by 1000 (horizontal) point
              bitmap. Each point represents display intensity for that screen location. Only CURVe?
              query functions are allowed on Fast Acquisition waveforms. When the
              ``WFMOutpre:PT_ORder?`` query returns Row, this indicates that the source is a Fast
              Acquisition waveform (and that each of 1000 possible horizontal columns being
              transmitted contains 500 vertical points. Note: 500 might vary based on how many
              channels enabled from same FastAcq group.) and waveform points are transmitted in the
              following order: left to right; then top to bottom. When the ``WFMOutpre:PT_ORder?``
              query returns Linear, this indicates that the source is not a Fast Acquisition
              waveform (and that each horizontal column being sent contains only one vertical
              point). Note that waveform points are transmitted in the following order: top to
              bottom, then left to right.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:PT_ORder?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_ORder?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:PT_ORder?
        """
        return self._pt_order

    @property
    def pt_off(self) -> WfmoutprePtOff:
        """Return the ``WFMOutpre:PT_Off`` command.

        **Description:**
            - This query-only command returns the trigger point relative to ``DATA:START`` for the
              waveform specified by the ``DATA:SOURCE`` command.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:PT_Off?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_Off?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:PT_Off?
        """
        return self._pt_off

    @property
    def reflevel(self) -> WfmoutpreReflevel:
        """Return the ``WFMOutpre:REFLEvel`` command.

        **Description:**
            - This query returns the Reference Level of the outgoing waveform. It applies only to
              the four frequency domain waveforms (RF Normal, RF Average, RF Max Hold, and RF Min
              Hold). The units are in watts.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:REFLEvel?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:REFLEvel?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:REFLEvel?
        """
        return self._reflevel

    @property
    def span(self) -> WfmoutpreSpan:
        """Return the ``WFMOutpre:SPAN`` command.

        **Description:**
            - This query returns the frequency span of the outgoing waveform. The span is the range
              of frequencies you can observe around the center frequency.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:SPAN?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:SPAN?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:SPAN?
        """
        return self._span

    @property
    def wfid(self) -> WfmoutpreWfid:
        """Return the ``WFMOutpre:WFId`` command.

        **Description:**
            - This query-only command returns a string describing several aspects of the acquisition
              parameters for the waveform specified by the ``DATA:SOURCE`` command. An error is
              reported if the ``DATa:SOUrce`` waveform does not exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:WFId?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:WFId?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:WFId?
        """
        return self._wfid

    @property
    def wfmtype(self) -> WfmoutpreWfmtype:
        """Return the ``WFMOutpre:WFMTYPe`` command.

        **Description:**
            - This query returns the type of the outgoing waveform. The type of waveform that is
              being transferred in turn determines in which window it will display on the instrument
              screen: (the Waveform View window or Spectrum View window). Both the Analog and
              ``RF_TD`` arguments specify the time domain window; the ``RF_RD`` argument specifies
              the frequency domain window.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:WFMTYPe?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:WFMTYPe?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:WFMTYPe?
        """
        return self._wfmtype

    @property
    def xincr(self) -> WfmoutpreXincr:
        """Return the ``WFMOutpre:XINcr`` command.

        **Description:**
            - This query-only command returns the horizontal point spacing in units of
              ``WFMOutpre:XUNit`` for the waveform specified by the ``DATA:SOURCE`` command. This
              value corresponds to the sampling interval. An error is reported if the
              ``DATa:SOUrce`` waveform does not exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:XINcr?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:XINcr?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:XINcr?
        """
        return self._xincr

    @property
    def xunit(self) -> WfmoutpreXunit:
        """Return the ``WFMOutpre:XUNit`` command.

        **Description:**
            - This query-only command returns the horizontal units for the waveform specified by the
              ``DATA:SOURCE`` command. An error is reported if the ``DATa:SOUrce`` waveform does not
              exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:XUNit?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:XUNit?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:XUNit?
        """
        return self._xunit

    @property
    def xzero(self) -> WfmoutpreXzero:
        """Return the ``WFMOutpre:XZEro`` command.

        **Description:**
            - This query-only command returns the sub-sample time between the trigger sample
              (designated by ``PT_OFF``) and the occurrence of the actual trigger for the waveform
              specified by the ``DATA:SOURCE`` command. This value is in units of
              ``WFMOutpre:XUNit``. An error is reported if the ``DATa:SOUrce`` waveform does not
              exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:XZEro?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:XZEro?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:XZEro?
        """
        return self._xzero

    @property
    def ymult(self) -> WfmoutpreYmult:
        """Return the ``WFMOutpre:YMUlt`` command.

        **Description:**
            - This query-only command returns the vertical scale factor per digitizing level in
              units specified by ``WFMOutpre:YUNit`` for the waveform specified by the
              ``DATA:SOURCE`` command. For those formats in which ``WFMOUTPRE:BYT_NR`` is important
              (all non-floating point formats), ``WFMOutpre:YMUlt?`` must take the location of the
              binary point implied by ``BYT_NR`` into consideration. An error is reported if the
              ``DATa:SOUrce`` waveform does not exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:YMUlt?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:YMUlt?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:YMUlt?
        """
        return self._ymult

    @property
    def yoff(self) -> WfmoutpreYoff:
        """Return the ``WFMOutpre:YOFf`` command.

        **Description:**
            - This query-only command returns the vertical offset of the source specified by
              ``DATA:SOURCE``. For this instrument family, the value returned is always 0.0 as the
              offset is combined with the ``:YZEro`` value. An error is reported if the
              ``DATA:SOURCE`` waveform does not exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:YOFf?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:YOFf?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:YOFf?
        """
        return self._yoff

    @property
    def yunit(self) -> WfmoutpreYunit:
        """Return the ``WFMOutpre:YUNit`` command.

        **Description:**
            - This query-only command returns the vertical units for the waveform specified by the
              ``DATA:SOURCE`` command. An error is reported if the ``DATa:SOUrce`` waveform does not
              exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:YUNit?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:YUNit?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:YUNit?
        """
        return self._yunit

    @property
    def yzero(self) -> WfmoutpreYzero:
        """Return the ``WFMOutpre:YZEro`` command.

        **Description:**
            - This query-only command returns the combined vertical position and offset for the
              source waveform specified by ``DATA:SOURCE``. This represents a departure from
              previous instruments where the ``:YZEro`` value represented the vertical position in
              vertical units and the ``:YOFf`` value represented the vertical offset in digitizing
              levels. For this instrument family, the value of ``:YOFf`` is always 0.0. An error is
              reported if the ``DATA:SOURCE`` waveform does not exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:YZEro?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:YZEro?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:YZEro?
        """
        return self._yzero



"""The verbose commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC, DSA70KD, LPD6, MSO2, MSO4, MSO5,
MSO5B, MSO5KB, MSO5LP, MSO6, MSO6B, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - VERBose {<NR1>|OFF|ON}
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class Verbose(SCPICmdWrite):
    """The ``VERBose`` command.

    **Description:**
        - This command sets or queries the Verbose state that controls the length of keywords on
          query responses. Keywords can be both headers and arguments.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``VERBose value`` command.

    **SCPI Syntax:**

    ::

        - VERBose {<NR1>|OFF|ON}

    **Info:**
        - ``<NR1>`` = 0 disables Verbose, any other value enables Verbose.
        - ``OFF`` sets the Verbose state to false, which returns minimum-length keywords for
          applicable setting queries.
        - ``ON`` sets the Verbose state to true, which returns full-length keywords for applicable
          setting queries.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "VERBose") -> None:
        super().__init__(device, cmd_syntax)



"""Base Application Programming Interface (API) device driver module."""
from abc import ABC

from tm_devices.drivers.device import Device


class APIDevice(Device, ABC):
    """Base Application Programming Interface (API) device driver."""



# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The digio commands module.

These commands are used in the following models:
SMU2604B, SMU2614B, SMU2634B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - digio.readbit()
    - digio.readport()
    - digio.trigger[N].assert()
    - digio.trigger[N].clear()
    - digio.trigger[N].mode
    - digio.trigger[N].overrun
    - digio.trigger[N].pulsewidth
    - digio.trigger[N].release()
    - digio.trigger[N].reset()
    - digio.trigger[N].stimulus
    - digio.trigger[N].wait()
    - digio.writebit()
    - digio.writeport()
    - digio.writeprotect
"""
from typing import Dict, Optional, TYPE_CHECKING, Union

from .._helpers import (
    BaseTSPCmd,
    DefaultDictPassKeyToFactory,
    NoDeviceProvidedError,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class DigioTriggerItem(ValidatedDynamicNumberCmd, BaseTSPCmd):
    """The ``digio.trigger[N]`` command tree.

    **Info:**
        - ``N``, the digital I/O trigger line (1 to 14).

    Constants:
        - ``.EVENT_ID``: The trigger event generated by the digital I/O line N.

    Properties/methods:
        - ``.assert()``: The ``digio.trigger[N].assert()`` function.
        - ``.clear()``: The ``digio.trigger[N].clear()`` function.
        - ``.mode``: The ``digio.trigger[N].mode`` attribute.
        - ``.overrun``: The ``digio.trigger[N].overrun`` attribute.
        - ``.pulsewidth``: The ``digio.trigger[N].pulsewidth`` attribute.
        - ``.release()``: The ``digio.trigger[N].release()`` function.
        - ``.reset()``: The ``digio.trigger[N].reset()`` function.
        - ``.stimulus``: The ``digio.trigger[N].stimulus`` attribute.
        - ``.wait()``: The ``digio.trigger[N].wait()`` function.
    """

    EVENT_ID = "digio.trigger[N].EVENT_ID"
    """str: The trigger event generated by the digital I/O line N."""

    def __init__(self, device: Optional["TSPDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        # pylint: disable=invalid-name
        self.EVENT_ID = self.EVENT_ID.replace(
            "[N]", f"[{self._cmd_syntax.rsplit('[', maxsplit=1)[-1].split(']', maxsplit=1)[0]}]"
        )

    @property
    def mode(self) -> str:
        """Access the ``digio.trigger[N].mode`` attribute.

        **Description:**
            - This attribute sets the mode in which the trigger event detector and the output
              trigger generator operate on the given trigger line. This command is not available on
              the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(digio.trigger[N].mode)`` query.
            - Setting this property to a value will send the ``digio.trigger[N].mode = value``
              command.

        **TSP Syntax:**

        ::

            - digio.trigger[N].mode = value
            - print(digio.trigger[N].mode)

        **Info:**
            - ``N``, the digital I/O trigger line (1 to 14).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".mode"
            return self._device.query(f"print({self._cmd_syntax}.mode)")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.mode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @mode.setter
    def mode(self, value: Union[str, float]) -> None:
        """Access the ``digio.trigger[N].mode`` attribute.

        **Description:**
            - This attribute sets the mode in which the trigger event detector and the output
              trigger generator operate on the given trigger line. This command is not available on
              the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(digio.trigger[N].mode)`` query.
            - Setting this property to a value will send the ``digio.trigger[N].mode = value``
              command.

        **TSP Syntax:**

        ::

            - digio.trigger[N].mode = value
            - print(digio.trigger[N].mode)

        **Info:**
            - ``N``, the digital I/O trigger line (1 to 14).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".mode", value
                )
            else:
                self._device.write(f"{self._cmd_syntax}.mode = {value}")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.mode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def overrun(self) -> str:
        """Access the ``digio.trigger[N].overrun`` attribute.

        **Description:**
            - This attribute returns the event detector overrun status. This command is not
              available on the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(digio.trigger[N].overrun)`` query.

        **TSP Syntax:**

        ::

            - print(digio.trigger[N].overrun)

        **Info:**
            - ``N``, the digital I/O trigger line (1 to 14).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".overrun"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.overrun)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.overrun`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def pulsewidth(self) -> str:
        """Access the ``digio.trigger[N].pulsewidth`` attribute.

        **Description:**
            - This attribute describes the length of time that the trigger line is asserted for
              output triggers. This command is not available on the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(digio.trigger[N].pulsewidth)`` query.
            - Setting this property to a value will send the ``digio.trigger[N].pulsewidth = value``
              command.

        **TSP Syntax:**

        ::

            - digio.trigger[N].pulsewidth = value
            - print(digio.trigger[N].pulsewidth)

        **Info:**
            - ``width``, the pulse width (seconds).
            - ``N``, the digital I/O trigger line (1 to 14).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".pulsewidth"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.pulsewidth)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.pulsewidth`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @pulsewidth.setter
    def pulsewidth(self, value: Union[str, float]) -> None:
        """Access the ``digio.trigger[N].pulsewidth`` attribute.

        **Description:**
            - This attribute describes the length of time that the trigger line is asserted for
              output triggers. This command is not available on the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(digio.trigger[N].pulsewidth)`` query.
            - Setting this property to a value will send the ``digio.trigger[N].pulsewidth = value``
              command.

        **TSP Syntax:**

        ::

            - digio.trigger[N].pulsewidth = value
            - print(digio.trigger[N].pulsewidth)

        **Info:**
            - ``width``, the pulse width (seconds).
            - ``N``, the digital I/O trigger line (1 to 14).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".pulsewidth", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.pulsewidth = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.pulsewidth`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def stimulus(self) -> str:
        """Access the ``digio.trigger[N].stimulus`` attribute.

        **Description:**
            - This attribute selects the event that causes a trigger to be asserted on the digital
              output line. This command is not available on the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(digio.trigger[N].stimulus)`` query.
            - Setting this property to a value will send the ``digio.trigger[N].stimulus = value``
              command.

        **TSP Syntax:**

        ::

            - digio.trigger[N].stimulus = value
            - print(digio.trigger[N].stimulus)

        **Info:**
            - ``N``, the digital I/O trigger line (1 to 14).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".stimulus"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.stimulus)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @stimulus.setter
    def stimulus(self, value: Union[str, float]) -> None:
        """Access the ``digio.trigger[N].stimulus`` attribute.

        **Description:**
            - This attribute selects the event that causes a trigger to be asserted on the digital
              output line. This command is not available on the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(digio.trigger[N].stimulus)`` query.
            - Setting this property to a value will send the ``digio.trigger[N].stimulus = value``
              command.

        **TSP Syntax:**

        ::

            - digio.trigger[N].stimulus = value
            - print(digio.trigger[N].stimulus)

        **Info:**
            - ``N``, the digital I/O trigger line (1 to 14).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".stimulus", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.stimulus = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def assert_(self) -> None:
        """Run the ``digio.trigger[N].assert()`` function.

        **Description:**
            - This function asserts a trigger pulse on one of the digital I/O lines. This command is
              not available on the 2604B, 2614B, or 2634B.

        **TSP Syntax:**

        ::

            - digio.trigger[N].assert()

        **Info:**
            - ``N``, the digital I/O trigger line (1 to 14).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.assert()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.assert()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def clear(self) -> None:
        """Run the ``digio.trigger[N].clear()`` function.

        **Description:**
            - This function clears the trigger event on a digital I/O line. This command is not
              available on the 2604B, 2614B, or 2634B.

        **TSP Syntax:**

        ::

            - digio.trigger[N].clear()

        **Info:**
            - ``N``, the digital I/O trigger line (1 to 14).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.clear()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.clear()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def release(self) -> None:
        """Run the ``digio.trigger[N].release()`` function.

        **Description:**
            - This function releases an indefinite length or latched trigger. This command is not
              available on the 2604B, 2614B, or 2634B.

        **TSP Syntax:**

        ::

            - digio.trigger[N].release()

        **Info:**
            - ``N``, the digital I/O trigger line (1 to 14).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.release()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.release()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def reset(self) -> None:
        """Run the ``digio.trigger[N].reset()`` function.

        **Description:**
            - This function resets trigger values to their factory defaults. This command is not
              available on the 2604B, 2614B, or 2634B.

        **TSP Syntax:**

        ::

            - digio.trigger[N].reset()

        **Info:**
            - ``N``, the digital I/O trigger line (1 to 14).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.reset()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.reset()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def wait(self, timeout: float) -> str:
        """Run the ``digio.trigger[N].wait()`` function.

        **Description:**
            - This function waits for a trigger. This command is not available on the 2604B, 2614B,
              or 2634B.

        **TSP Syntax:**

        ::

            - digio.trigger[N].wait()

        Args:
            timeout: Timeout in seconds.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.wait({timeout}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.wait()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class Digio(BaseTSPCmd):
    """The ``digio`` command tree.

    Properties/methods:
        - ``.readbit()``: The ``digio.readbit()`` function.
        - ``.readport()``: The ``digio.readport()`` function.
        - ``.trigger``: The ``digio.trigger[N]`` command tree.
        - ``.writebit()``: The ``digio.writebit()`` function.
        - ``.writeport()``: The ``digio.writeport()`` function.
        - ``.writeprotect``: The ``digio.writeprotect`` attribute.
    """

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "digio") -> None:
        super().__init__(device, cmd_syntax)
        self._trigger: Dict[int, DigioTriggerItem] = DefaultDictPassKeyToFactory(
            lambda x: DigioTriggerItem(device, f"{self._cmd_syntax}.trigger[{x}]")
        )

    @property
    def trigger(self) -> Dict[int, DigioTriggerItem]:
        """Return the ``digio.trigger[N]`` command tree.

        **Info:**
            - ``N``, the digital I/O trigger line (1 to 14).

        Constants:
            - ``.EVENT_ID``: The trigger event generated by the digital I/O line N.

        Sub-properties/methods:
            - ``.assert()``: The ``digio.trigger[N].assert()`` function.
            - ``.clear()``: The ``digio.trigger[N].clear()`` function.
            - ``.mode``: The ``digio.trigger[N].mode`` attribute.
            - ``.overrun``: The ``digio.trigger[N].overrun`` attribute.
            - ``.pulsewidth``: The ``digio.trigger[N].pulsewidth`` attribute.
            - ``.release()``: The ``digio.trigger[N].release()`` function.
            - ``.reset()``: The ``digio.trigger[N].reset()`` function.
            - ``.stimulus``: The ``digio.trigger[N].stimulus`` attribute.
            - ``.wait()``: The ``digio.trigger[N].wait()`` function.
        """
        return self._trigger

    @property
    def writeprotect(self) -> str:
        """Access the ``digio.writeprotect`` attribute.

        **Description:**
            - This attribute contains the write-protect mask that protects bits from changes from
              the digio.writebit() and digio.writeport() functions. This command is not available on
              the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(digio.writeprotect)`` query.
            - Setting this property to a value will send the ``digio.writeprotect = value`` command.

        **TSP Syntax:**

        ::

            - digio.writeprotect = value
            - print(digio.writeprotect)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".writeprotect"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.writeprotect)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.writeprotect`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @writeprotect.setter
    def writeprotect(self, value: Union[str, float]) -> None:
        """Access the ``digio.writeprotect`` attribute.

        **Description:**
            - This attribute contains the write-protect mask that protects bits from changes from
              the digio.writebit() and digio.writeport() functions. This command is not available on
              the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(digio.writeprotect)`` query.
            - Setting this property to a value will send the ``digio.writeprotect = value`` command.

        **TSP Syntax:**

        ::

            - digio.writeprotect = value
            - print(digio.writeprotect)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".writeprotect", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.writeprotect = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.writeprotect`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def readbit(self, n: int) -> str:
        """Run the ``digio.readbit()`` function.

        **Description:**
            - This function reads one digital I/O line. This command is not available on the 2604B,
              2614B, or 2634B.

        **TSP Syntax:**

        ::

            - digio.readbit()

        Args:
            n: Digital I/O line number to be read (1 to 14).

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.readbit({n}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.readbit()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def readport(self) -> str:
        """Run the ``digio.readport()`` function.

        **Description:**
            - This function reads the digital I/O port. This command is not available on the 2604B,
              2614B, or 2634B.

        **TSP Syntax:**

        ::

            - digio.readport()

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.readport())"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.readport()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def writebit(self, n: int, data: int) -> None:
        """Run the ``digio.writebit()`` function.

        **Description:**
            - This function sets a digital I/O line high or low. This command is not available on
              the 2604B, 2614B, or 2634B.

        **TSP Syntax:**

        ::

            - digio.writebit()

        Args:
            n: Digital I/O trigger line (1 to 14).
            data: The value to write to the bit.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.writebit({n}, {data})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.writebit()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def writeport(self, data: int) -> None:
        """Run the ``digio.writeport()`` function.

        **Description:**
            - This function writes to all digital I/O lines. This command is not available on the
              2604B, 2614B, or 2634B.

        **TSP Syntax:**

        ::

            - digio.writeport()

        Args:
            data: Value to write to the port (0 to 16383).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.writeport({data})")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.writeport()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The autosavepitimeout commands module.

These commands are used in the following models:
LPD6, MSO2, MSO4, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - AUTOSAVEPITIMEOUT <NR1>
    - AUTOSAVEPITIMEOUT?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class Autosavepitimeout(SCPICmdWrite, SCPICmdRead):
    """The ``AUTOSAVEPITIMEOUT`` command.

    **Description:**
        - This command sets or queries the idle time from the programmable interface before
          auto-save occurs.

    **Usage:**
        - Using the ``.query()`` method will send the ``AUTOSAVEPITIMEOUT?`` query.
        - Using the ``.verify(value)`` method will send the ``AUTOSAVEPITIMEOUT?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUTOSAVEPITIMEOUT value`` command.

    **SCPI Syntax:**

    ::

        - AUTOSAVEPITIMEOUT <NR1>
        - AUTOSAVEPITIMEOUT?

    **Info:**
        - ``<NR1>``
    """

    def __init__(
        self, device: Optional["PIDevice"] = None, cmd_syntax: str = "AUTOSAVEPITIMEOUT"
    ) -> None:
        super().__init__(device, cmd_syntax)



# pylint: disable=line-too-long
"""The mark commands module.

These commands are used in the following models:
DPO4K, DPO4KB, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - MARK {NEXT|PREVious}
    - MARK:CREATE {CH<x>|MATH|REF1 |REF2|REF3|REF4|B1|B2|B3|B4|REF1|REF2|REF3|REF4|DIGital| COLUMN|RF_AMPlitude|RF_FREQuency|RF_PHASe}
    - MARK:DELEte {CH<x>|MATH|REF1|REF2|REF3|REF4|B1|B2|B3|B4|REF1|REF2| REF3|REF4|DIGital|COLUMN|SELECTED|ALL||RF_AMPlitude|RF_FREQuency| RF_PHASe}
    - MARK:FREE?
    - MARK:SAVEALL TOUSER
    - MARK:SELected:END?
    - MARK:SELected:FOCUS?
    - MARK:SELected:MARKSINCOLumn?
    - MARK:SELected:OWNer?
    - MARK:SELected:SOURCe?
    - MARK:SELected:STARt?
    - MARK:SELected:STATE?
    - MARK:SELected:ZOOm:POSition?
    - MARK:TOTal?
    - MARK:USERLIST <Enum>,<NR3>,<NR3>,<NR3>,<NR3>,<NR3>,<NR3>,<NR3>
    - MARK:USERLIST?
    - MARK?
"""  # noqa: E501
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class MarkUserlist(SCPICmdWrite, SCPICmdRead):
    """The ``MARK:USERLIST`` command.

    **Description:**
        - The command creates a single user mark on a waveform in the time domain. The arguments
          consist of an enumeration specifying the source waveform, followed by 7 time mark
          parameters. You can create up to 1,024 marks. To save all the marks to memory, use the
          command ``MARK:SAVEALL TOUSER``. The query form retrieves a list of all user marks,
          separated by a semicolon. To retrieve the list of all system generated marks, use the
          command ``SEARCH:SEARCHX:LIST A`` 'Settings conflict' error event is set for the command
          form if any of the following conditions are true: The source waveform is not turned on.
          The position of the mark is not on the waveform. The maximum number of marks would be
          exceeded.

    **Usage:**
        - Using the ``.query()`` method will send the ``MARK:USERLIST?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:USERLIST?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MARK:USERLIST value`` command.

    **SCPI Syntax:**

    ::

        - MARK:USERLIST <Enum>,<NR3>,<NR3>,<NR3>,<NR3>,<NR3>,<NR3>,<NR3>
        - MARK:USERLIST?

    **Info:**
        - ``CH<x>`` - analog channels 1-4.
        - ``B<x>`` - serial (if installed) or parallel bus 1-4.
        - ``MATH`` - math waveform.
        - ``REF<x>`` - reference waveforms 1-4.
        - ``D0 - D15`` - digital channels 0 - 15 (Requires installation of option 3-MSO.).
    """


class MarkTotal(SCPICmdRead):
    """The ``MARK:TOTal`` command.

    **Description:**
        - Returns how many marks are currently in use. There can be a total of 1,024 marks returned.

    **Usage:**
        - Using the ``.query()`` method will send the ``MARK:TOTal?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:TOTal?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - MARK:TOTal?
    """


class MarkSelectedZoomPosition(SCPICmdRead):
    """The ``MARK:SELected:ZOOm:POSition`` command.

    **Description:**
        - Returns the position of the selected mark, 0 to 100% of the zoom overview window.

    **Usage:**
        - Using the ``.query()`` method will send the ``MARK:SELected:ZOOm:POSition?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:ZOOm:POSition?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - MARK:SELected:ZOOm:POSition?
    """


class MarkSelectedZoom(SCPICmdRead):
    """The ``MARK:SELected:ZOOm`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``MARK:SELected:ZOOm?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:ZOOm?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.position``: The ``MARK:SELected:ZOOm:POSition`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._position = MarkSelectedZoomPosition(device, f"{self._cmd_syntax}:POSition")

    @property
    def position(self) -> MarkSelectedZoomPosition:
        """Return the ``MARK:SELected:ZOOm:POSition`` command.

        **Description:**
            - Returns the position of the selected mark, 0 to 100% of the zoom overview window.

        **Usage:**
            - Using the ``.query()`` method will send the ``MARK:SELected:ZOOm:POSition?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:ZOOm:POSition?``
              query and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - MARK:SELected:ZOOm:POSition?
        """
        return self._position


class MarkSelectedState(SCPICmdRead):
    """The ``MARK:SELected:STATE`` command.

    **Description:**
        - Returns the on or off state of the selected mark. The selected mark is at or near the
          center of the screen. If you press the front-panel Set/Clear button, this mark will
          disappear.

    **Usage:**
        - Using the ``.query()`` method will send the ``MARK:SELected:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:STATE?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - MARK:SELected:STATE?
    """


class MarkSelectedStart(SCPICmdRead):
    """The ``MARK:SELected:STARt`` command.

    **Description:**
        - Returns the starting point of the selected mark, 0 to 100% of the waveform.

    **Usage:**
        - Using the ``.query()`` method will send the ``MARK:SELected:STARt?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:STARt?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - MARK:SELected:STARt?
    """


class MarkSelectedSource(SCPICmdRead):
    """The ``MARK:SELected:SOURCe`` command.

    **Description:**
        - Returns the source waveform for the selected mark.

    **Usage:**
        - Using the ``.query()`` method will send the ``MARK:SELected:SOURCe?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:SOURCe?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - MARK:SELected:SOURCe?
    """


class MarkSelectedOwner(SCPICmdRead):
    """The ``MARK:SELected:OWNer`` command.

    **Description:**
        - Returns the owner of the selected mark.

    **Usage:**
        - Using the ``.query()`` method will send the ``MARK:SELected:OWNer?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:OWNer?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - MARK:SELected:OWNer?
    """


class MarkSelectedMarksincolumn(SCPICmdRead):
    """The ``MARK:SELected:MARKSINCOLumn`` command.

    **Description:**
        - Returns the number of marks in the current zoom pixel column.

    **Usage:**
        - Using the ``.query()`` method will send the ``MARK:SELected:MARKSINCOLumn?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:MARKSINCOLumn?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - MARK:SELected:MARKSINCOLumn?
    """


class MarkSelectedFocus(SCPICmdRead):
    """The ``MARK:SELected:FOCUS`` command.

    **Description:**
        - Returns the focus of the selected mark, 0 to 100% of the waveform.

    **Usage:**
        - Using the ``.query()`` method will send the ``MARK:SELected:FOCUS?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:FOCUS?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - MARK:SELected:FOCUS?
    """


class MarkSelectedEnd(SCPICmdRead):
    """The ``MARK:SELected:END`` command.

    **Description:**
        - Returns the end of the selected mark, 0 to 100% of the waveform.

    **Usage:**
        - Using the ``.query()`` method will send the ``MARK:SELected:END?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:END?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - MARK:SELected:END?
    """


#  pylint: disable=too-many-instance-attributes
class MarkSelected(SCPICmdRead):
    """The ``MARK:SELected`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``MARK:SELected?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.end``: The ``MARK:SELected:END`` command.
        - ``.focus``: The ``MARK:SELected:FOCUS`` command.
        - ``.marksincolumn``: The ``MARK:SELected:MARKSINCOLumn`` command.
        - ``.owner``: The ``MARK:SELected:OWNer`` command.
        - ``.source``: The ``MARK:SELected:SOURCe`` command.
        - ``.start``: The ``MARK:SELected:STARt`` command.
        - ``.state``: The ``MARK:SELected:STATE`` command.
        - ``.zoom``: The ``MARK:SELected:ZOOm`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._end = MarkSelectedEnd(device, f"{self._cmd_syntax}:END")
        self._focus = MarkSelectedFocus(device, f"{self._cmd_syntax}:FOCUS")
        self._marksincolumn = MarkSelectedMarksincolumn(device, f"{self._cmd_syntax}:MARKSINCOLumn")
        self._owner = MarkSelectedOwner(device, f"{self._cmd_syntax}:OWNer")
        self._source = MarkSelectedSource(device, f"{self._cmd_syntax}:SOURCe")
        self._start = MarkSelectedStart(device, f"{self._cmd_syntax}:STARt")
        self._state = MarkSelectedState(device, f"{self._cmd_syntax}:STATE")
        self._zoom = MarkSelectedZoom(device, f"{self._cmd_syntax}:ZOOm")

    @property
    def end(self) -> MarkSelectedEnd:
        """Return the ``MARK:SELected:END`` command.

        **Description:**
            - Returns the end of the selected mark, 0 to 100% of the waveform.

        **Usage:**
            - Using the ``.query()`` method will send the ``MARK:SELected:END?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:END?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - MARK:SELected:END?
        """
        return self._end

    @property
    def focus(self) -> MarkSelectedFocus:
        """Return the ``MARK:SELected:FOCUS`` command.

        **Description:**
            - Returns the focus of the selected mark, 0 to 100% of the waveform.

        **Usage:**
            - Using the ``.query()`` method will send the ``MARK:SELected:FOCUS?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:FOCUS?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - MARK:SELected:FOCUS?
        """
        return self._focus

    @property
    def marksincolumn(self) -> MarkSelectedMarksincolumn:
        """Return the ``MARK:SELected:MARKSINCOLumn`` command.

        **Description:**
            - Returns the number of marks in the current zoom pixel column.

        **Usage:**
            - Using the ``.query()`` method will send the ``MARK:SELected:MARKSINCOLumn?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:MARKSINCOLumn?``
              query and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - MARK:SELected:MARKSINCOLumn?
        """
        return self._marksincolumn

    @property
    def owner(self) -> MarkSelectedOwner:
        """Return the ``MARK:SELected:OWNer`` command.

        **Description:**
            - Returns the owner of the selected mark.

        **Usage:**
            - Using the ``.query()`` method will send the ``MARK:SELected:OWNer?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:OWNer?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - MARK:SELected:OWNer?
        """
        return self._owner

    @property
    def source(self) -> MarkSelectedSource:
        """Return the ``MARK:SELected:SOURCe`` command.

        **Description:**
            - Returns the source waveform for the selected mark.

        **Usage:**
            - Using the ``.query()`` method will send the ``MARK:SELected:SOURCe?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:SOURCe?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - MARK:SELected:SOURCe?
        """
        return self._source

    @property
    def start(self) -> MarkSelectedStart:
        """Return the ``MARK:SELected:STARt`` command.

        **Description:**
            - Returns the starting point of the selected mark, 0 to 100% of the waveform.

        **Usage:**
            - Using the ``.query()`` method will send the ``MARK:SELected:STARt?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:STARt?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - MARK:SELected:STARt?
        """
        return self._start

    @property
    def state(self) -> MarkSelectedState:
        """Return the ``MARK:SELected:STATE`` command.

        **Description:**
            - Returns the on or off state of the selected mark. The selected mark is at or near the
              center of the screen. If you press the front-panel Set/Clear button, this mark will
              disappear.

        **Usage:**
            - Using the ``.query()`` method will send the ``MARK:SELected:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:STATE?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - MARK:SELected:STATE?
        """
        return self._state

    @property
    def zoom(self) -> MarkSelectedZoom:
        """Return the ``MARK:SELected:ZOOm`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``MARK:SELected:ZOOm?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:ZOOm?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.position``: The ``MARK:SELected:ZOOm:POSition`` command.
        """
        return self._zoom


class MarkSaveall(SCPICmdWrite):
    """The ``MARK:SAVEALL`` command.

    **Description:**
        - This command saves all current marks on waveforms in the time domain to the user search
          mark list in internal memory. (This is equivalent to pressing the 'Save All Marks' button
          in the Search button menu on the front panel.) In order to retrieve the information, use
          the query form of ``MARK:USERLIST``.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``MARK:SAVEALL value`` command.

    **SCPI Syntax:**

    ::

        - MARK:SAVEALL TOUSER
    """


class MarkFree(SCPICmdRead):
    """The ``MARK:FREE`` command.

    **Description:**
        - Returns how many marks are available for use. There can be a total of 1,024 marks
          returned.

    **Usage:**
        - Using the ``.query()`` method will send the ``MARK:FREE?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:FREE?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - MARK:FREE?
    """


class MarkDelete(SCPICmdWrite):
    """The ``MARK:DELEte`` command.

    **Description:**
        - This command deletes a mark on a particular waveform, all waveforms in a column, the
          selected mark, or all marks.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``MARK:DELEte value`` command.

    **SCPI Syntax:**

    ::

        - MARK:DELEte {CH<x>|MATH|REF1|REF2|REF3|REF4|B1|B2|B3|B4|REF1|REF2| REF3|REF4|DIGital|COLUMN|SELECTED|ALL||RF_AMPlitude|RF_FREQuency| RF_PHASe}

    **Info:**
        - ``CH<x>`` deletes the mark on a channel waveform, where <x> is the channel number.
        - ``MATH`` deletes the mark on the math waveform.
        - ``B<x>`` deletes the mark on a bus waveform, where <x>.
        - ``REF<x>`` deletes the mark on a reference waveform, where <x> is the reference waveform
          number.
        - ``DIGital`` deletes all marks on all digital channels. (Requires installation of option
          3-MSO.).
        - ``COLUMN`` deletes marks on all waveforms in the current zoom pixel column.
    """  # noqa: E501


class MarkCreate(SCPICmdWrite):
    """The ``MARK:CREATE`` command.

    **Description:**
        - Creates a mark on a specified waveform or all waveforms in a column.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``MARK:CREATE value`` command.

    **SCPI Syntax:**

    ::

        - MARK:CREATE {CH<x>|MATH|REF1 |REF2|REF3|REF4|B1|B2|B3|B4|REF1|REF2|REF3|REF4|DIGital| COLUMN|RF_AMPlitude|RF_FREQuency|RF_PHASe}

    **Info:**
        - ``CH<x>`` creates the mark on a channel waveform, where <x> is the channel number.
        - ``MATH`` creates the mark on the math waveform.
        - ``B<x>`` creates the mark on a bus waveform, where <x>.
        - ``REF<x>`` creates the mark on a reference waveform, where <x> is the reference waveform
          number.
        - ``DIGital`` creates the mark on a digital waveform. (An error will result if no digital
          channel is turned on.) (Requires installation of option 3-MSO.).
        - ``COLUMN`` creates marks on all waveforms in the current zoom pixel column.
    """  # noqa: E501


class Mark(SCPICmdWrite, SCPICmdRead):
    """The ``MARK`` command.

    **Description:**
        - Moves to the next or previous reference mark on the waveform. Returns the current mark
          position.

    **Usage:**
        - Using the ``.query()`` method will send the ``MARK?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MARK value`` command.

    **SCPI Syntax:**

    ::

        - MARK {NEXT|PREVious}
        - MARK?

    **Info:**
        - ``NEXT`` moves to the next reference mark on the right.
        - ``PREVious`` moves to the next reference mark on the left.

    Properties:
        - ``.create``: The ``MARK:CREATE`` command.
        - ``.delete``: The ``MARK:DELEte`` command.
        - ``.free``: The ``MARK:FREE`` command.
        - ``.saveall``: The ``MARK:SAVEALL`` command.
        - ``.selected``: The ``MARK:SELected`` command tree.
        - ``.total``: The ``MARK:TOTal`` command.
        - ``.userlist``: The ``MARK:USERLIST`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "MARK") -> None:
        super().__init__(device, cmd_syntax)
        self._create = MarkCreate(device, f"{self._cmd_syntax}:CREATE")
        self._delete = MarkDelete(device, f"{self._cmd_syntax}:DELEte")
        self._free = MarkFree(device, f"{self._cmd_syntax}:FREE")
        self._saveall = MarkSaveall(device, f"{self._cmd_syntax}:SAVEALL")
        self._selected = MarkSelected(device, f"{self._cmd_syntax}:SELected")
        self._total = MarkTotal(device, f"{self._cmd_syntax}:TOTal")
        self._userlist = MarkUserlist(device, f"{self._cmd_syntax}:USERLIST")

    @property
    def create(self) -> MarkCreate:
        """Return the ``MARK:CREATE`` command.

        **Description:**
            - Creates a mark on a specified waveform or all waveforms in a column.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``MARK:CREATE value`` command.

        **SCPI Syntax:**

        ::

            - MARK:CREATE {CH<x>|MATH|REF1 |REF2|REF3|REF4|B1|B2|B3|B4|REF1|REF2|REF3|REF4|DIGital| COLUMN|RF_AMPlitude|RF_FREQuency|RF_PHASe}

        **Info:**
            - ``CH<x>`` creates the mark on a channel waveform, where <x> is the channel number.
            - ``MATH`` creates the mark on the math waveform.
            - ``B<x>`` creates the mark on a bus waveform, where <x>.
            - ``REF<x>`` creates the mark on a reference waveform, where <x> is the reference
              waveform number.
            - ``DIGital`` creates the mark on a digital waveform. (An error will result if no
              digital channel is turned on.) (Requires installation of option 3-MSO.).
            - ``COLUMN`` creates marks on all waveforms in the current zoom pixel column.
        """  # noqa: E501
        return self._create

    @property
    def delete(self) -> MarkDelete:
        """Return the ``MARK:DELEte`` command.

        **Description:**
            - This command deletes a mark on a particular waveform, all waveforms in a column, the
              selected mark, or all marks.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``MARK:DELEte value`` command.

        **SCPI Syntax:**

        ::

            - MARK:DELEte {CH<x>|MATH|REF1|REF2|REF3|REF4|B1|B2|B3|B4|REF1|REF2| REF3|REF4|DIGital|COLUMN|SELECTED|ALL||RF_AMPlitude|RF_FREQuency| RF_PHASe}

        **Info:**
            - ``CH<x>`` deletes the mark on a channel waveform, where <x> is the channel number.
            - ``MATH`` deletes the mark on the math waveform.
            - ``B<x>`` deletes the mark on a bus waveform, where <x>.
            - ``REF<x>`` deletes the mark on a reference waveform, where <x> is the reference
              waveform number.
            - ``DIGital`` deletes all marks on all digital channels. (Requires installation of
              option 3-MSO.).
            - ``COLUMN`` deletes marks on all waveforms in the current zoom pixel column.
        """  # noqa: E501
        return self._delete

    @property
    def free(self) -> MarkFree:
        """Return the ``MARK:FREE`` command.

        **Description:**
            - Returns how many marks are available for use. There can be a total of 1,024 marks
              returned.

        **Usage:**
            - Using the ``.query()`` method will send the ``MARK:FREE?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:FREE?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - MARK:FREE?
        """
        return self._free

    @property
    def saveall(self) -> MarkSaveall:
        """Return the ``MARK:SAVEALL`` command.

        **Description:**
            - This command saves all current marks on waveforms in the time domain to the user
              search mark list in internal memory. (This is equivalent to pressing the 'Save All
              Marks' button in the Search button menu on the front panel.) In order to retrieve the
              information, use the query form of ``MARK:USERLIST``.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``MARK:SAVEALL value`` command.

        **SCPI Syntax:**

        ::

            - MARK:SAVEALL TOUSER
        """
        return self._saveall

    @property
    def selected(self) -> MarkSelected:
        """Return the ``MARK:SELected`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``MARK:SELected?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.end``: The ``MARK:SELected:END`` command.
            - ``.focus``: The ``MARK:SELected:FOCUS`` command.
            - ``.marksincolumn``: The ``MARK:SELected:MARKSINCOLumn`` command.
            - ``.owner``: The ``MARK:SELected:OWNer`` command.
            - ``.source``: The ``MARK:SELected:SOURCe`` command.
            - ``.start``: The ``MARK:SELected:STARt`` command.
            - ``.state``: The ``MARK:SELected:STATE`` command.
            - ``.zoom``: The ``MARK:SELected:ZOOm`` command tree.
        """
        return self._selected

    @property
    def total(self) -> MarkTotal:
        """Return the ``MARK:TOTal`` command.

        **Description:**
            - Returns how many marks are currently in use. There can be a total of 1,024 marks
              returned.

        **Usage:**
            - Using the ``.query()`` method will send the ``MARK:TOTal?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:TOTal?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - MARK:TOTal?
        """
        return self._total

    @property
    def userlist(self) -> MarkUserlist:
        """Return the ``MARK:USERLIST`` command.

        **Description:**
            - The command creates a single user mark on a waveform in the time domain. The arguments
              consist of an enumeration specifying the source waveform, followed by 7 time mark
              parameters. You can create up to 1,024 marks. To save all the marks to memory, use the
              command ``MARK:SAVEALL TOUSER``. The query form retrieves a list of all user marks,
              separated by a semicolon. To retrieve the list of all system generated marks, use the
              command ``SEARCH:SEARCHX:LIST A`` 'Settings conflict' error event is set for the
              command form if any of the following conditions are true: The source waveform is not
              turned on. The position of the mark is not on the waveform. The maximum number of
              marks would be exceeded.

        **Usage:**
            - Using the ``.query()`` method will send the ``MARK:USERLIST?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:USERLIST?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MARK:USERLIST value`` command.

        **SCPI Syntax:**

        ::

            - MARK:USERLIST <Enum>,<NR3>,<NR3>,<NR3>,<NR3>,<NR3>,<NR3>,<NR3>
            - MARK:USERLIST?

        **Info:**
            - ``CH<x>`` - analog channels 1-4.
            - ``B<x>`` - serial (if installed) or parallel bus 1-4.
            - ``MATH`` - math waveform.
            - ``REF<x>`` - reference waveforms 1-4.
            - ``D0 - D15`` - digital channels 0 - 15 (Requires installation of option 3-MSO.).
        """
        return self._userlist



"""The source3 commands module.

These commands are used in the following models:
AFG3K, AFG3KB, AFG3KC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - SOURce3:POWer:LEVel:IMMediate:AMPLitude {<percent>|MINimum|MAXimum}
    - SOURce3:POWer:LEVel:IMMediate:AMPLitude?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class Source3PowerLevelImmediateAmplitude(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce3:POWer:LEVel:IMMediate:AMPLitude`` command.

    **Description:**
        - This command sets or queries the internal noise level which applies to the output signal
          for the specified channel. The noise level represents the percent against current
          amplitude level. The setting range is 0 to 50%. This command is available when Run Mode is
          set to Continuous, Burst, or Sweep. You can set or query whether to add the internal noise
          to the output signal using the [SOURce[1|2]]``:COMBine:FEED`` command.

    **Usage:**
        - Using the ``.query()`` method will send the ``SOURce3:POWer:LEVel:IMMediate:AMPLitude?``
          query.
        - Using the ``.verify(value)`` method will send the
          ``SOURce3:POWer:LEVel:IMMediate:AMPLitude?`` query and raise an AssertionError if the
          returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SOURce3:POWer:LEVel:IMMediate:AMPLitude value`` command.

    **SCPI Syntax:**

    ::

        - SOURce3:POWer:LEVel:IMMediate:AMPLitude {<percent>|MINimum|MAXimum}
        - SOURce3:POWer:LEVel:IMMediate:AMPLitude?
    """


class Source3PowerLevelImmediate(SCPICmdRead):
    """The ``SOURce3:POWer:LEVel:IMMediate`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SOURce3:POWer:LEVel:IMMediate?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce3:POWer:LEVel:IMMediate?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.amplitude``: The ``SOURce3:POWer:LEVel:IMMediate:AMPLitude`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._amplitude = Source3PowerLevelImmediateAmplitude(
            device, f"{self._cmd_syntax}:AMPLitude"
        )

    @property
    def amplitude(self) -> Source3PowerLevelImmediateAmplitude:
        """Return the ``SOURce3:POWer:LEVel:IMMediate:AMPLitude`` command.

        **Description:**
            - This command sets or queries the internal noise level which applies to the output
              signal for the specified channel. The noise level represents the percent against
              current amplitude level. The setting range is 0 to 50%. This command is available when
              Run Mode is set to Continuous, Burst, or Sweep. You can set or query whether to add
              the internal noise to the output signal using the [SOURce[1|2]]``:COMBine:FEED``
              command.

        **Usage:**
            - Using the ``.query()`` method will send the
              ``SOURce3:POWer:LEVel:IMMediate:AMPLitude?`` query.
            - Using the ``.verify(value)`` method will send the
              ``SOURce3:POWer:LEVel:IMMediate:AMPLitude?`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SOURce3:POWer:LEVel:IMMediate:AMPLitude value`` command.

        **SCPI Syntax:**

        ::

            - SOURce3:POWer:LEVel:IMMediate:AMPLitude {<percent>|MINimum|MAXimum}
            - SOURce3:POWer:LEVel:IMMediate:AMPLitude?
        """
        return self._amplitude


class Source3PowerLevel(SCPICmdRead):
    """The ``SOURce3:POWer:LEVel`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SOURce3:POWer:LEVel?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce3:POWer:LEVel?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.immediate``: The ``SOURce3:POWer:LEVel:IMMediate`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._immediate = Source3PowerLevelImmediate(device, f"{self._cmd_syntax}:IMMediate")

    @property
    def immediate(self) -> Source3PowerLevelImmediate:
        """Return the ``SOURce3:POWer:LEVel:IMMediate`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SOURce3:POWer:LEVel:IMMediate?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce3:POWer:LEVel:IMMediate?``
              query and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.amplitude``: The ``SOURce3:POWer:LEVel:IMMediate:AMPLitude`` command.
        """
        return self._immediate


class Source3Power(SCPICmdRead):
    """The ``SOURce3:POWer`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SOURce3:POWer?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce3:POWer?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.level``: The ``SOURce3:POWer:LEVel`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._level = Source3PowerLevel(device, f"{self._cmd_syntax}:LEVel")

    @property
    def level(self) -> Source3PowerLevel:
        """Return the ``SOURce3:POWer:LEVel`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SOURce3:POWer:LEVel?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce3:POWer:LEVel?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.immediate``: The ``SOURce3:POWer:LEVel:IMMediate`` command tree.
        """
        return self._level


class Source3(SCPICmdRead):
    """The ``SOURce3`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SOURce3?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce3?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.power``: The ``SOURce3:POWer`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "SOURce3") -> None:
        super().__init__(device, cmd_syntax)
        self._power = Source3Power(device, f"{self._cmd_syntax}:POWer")

    @property
    def power(self) -> Source3Power:
        """Return the ``SOURce3:POWer`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SOURce3:POWer?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce3:POWer?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.level``: The ``SOURce3:POWer:LEVel`` command tree.
        """
        return self._power
